{% for f in items | natsort("identifier") %}
{{ f.identifier | MkHeader }}
{{ (f.identifier ~ f.filter_fn | format_signature) | md_style(code=True) }}

{{ f.filter_fn | get_doc(only_summary=True) }}

{% if f.aliases %}
**Aliases:** {% for alias in f.aliases %} `{{ alias }}` {% endfor %}
{% endif %}
{% if f.required_packages %}
**Required packages:** {% for required_package in f.required_packages %} `{{ required_package}}` {% endfor %}
{% endif %}

{% for k, v in f.examples.items() %}

{% filter MkAdmonition(title="Example") | string %}
Jinja call:
{{ v.template | MkCode(language="jinja") }}
{% if mode == "tests" or v.markdown is defined %}
Result:
{{ v.template | render_string | MkCode(language="") }}
{% else %}
Result: {{ v.template | render_string | md_style(code=True) }}
{% endif %}
{% endfilter %}
{% endfor %}
{{ f.filter_fn
   | MkDocStrings(show_docstring_description=False, heading_level=4, filters=["!^_"])
   | MkAdmonition(collapsible=True, title="DocStrings")
}}


{% endfor %}
